with Ada.Text_IO; use Ada.Text_IO;
with Ada.Strings.Bounded;
use Ada.Strings.Bounded;

with Terminal_Interface.Curses; use Terminal_Interface.Curses;
with Terminal_Interface.Curses.Aux; use Terminal_Interface.Curses.Aux;

with Interfaces.C; use Interfaces.C;
with Interfaces.C.Strings; use Interfaces.C.Strings;


package body ncurses2.genericPuts is

	procedure myGet (Win : in  Window := Standard_Window;
				  Str : out BS.Bounded_String;
				  Len : in  Integer := -1)
	is
		use BS;
		function Wgetnstr (Win : Window;
						 Str : char_array;
						 Len : int) return int;
		pragma Import (C, Wgetnstr, "wgetnstr");

		N	 : Integer := Len;
		Txt : char_array (0 .. size_t(Max_Length));
		xStr : String(1 .. Max_Length);
		Cnt : Natural;
	 begin
		if N < 0 then
			N := Max_Length;
		end if;
		if N > Max_Length then
		 raise Constraint_Error;
		end if;
		Txt (0) := Interfaces.C.char'First;
		if Wgetnstr (Win, Txt, C_Int (N)) = Curses_Err then
		 raise Curses_Exception;
		end if;
		To_Ada (Txt, xStr, Cnt, True);
		Str := To_Bounded_String(xStr(1..Cnt));
	end myGet;



	procedure myPut (str : out BS.Bounded_String; i : Integer;
		Base : in     Number_Base := 10) is ----- Default_Base) is 
		package Int_IO is new Integer_IO(Integer); use Int_IO;
		tmp : String(1.. BS.Max_Length);	
	begin
		Put(tmp, i, Base);
		str := To_Bounded_String(tmp);
		Trim(str, Ada.Strings.Trim_End'(Ada.Strings.Left));
		-- str := Trim(tmp, Ada.Strings.Trim_End'(Ada.Strings.Left));
	end myPut;

	procedure myAdd (Str : BS.Bounded_String) is
	begin
		Add(Str => To_String(Str));
	end;

	-- from ncurses-aux
 	procedure Fill_String (Cp  : in  chars_ptr;
						 Str : out BS.Bounded_String)
	is
		--  Fill the string with the characters referenced by the
		--  chars_ptr.
		--
		Len : Natural;
	begin
		if Cp /= Null_Ptr then
			Len := Natural (Strlen (Cp));
			if Max_Length < Len then
				raise Constraint_Error;
			end if;
			declare
				S : String (1 .. Len);
			begin
				S := Value (Cp);
				Str := To_Bounded_String(S);
			end;
		else
			Str := Null_Bounded_String;
		end if;

	end Fill_String;

end ncurses2.GenericPuts;
