--  A simplified version of the  GNU getopt function
--  copyright Free Software Foundtion

with Ada.Strings.Fixed;
with Ada.Strings.Bounded;
with Ada.Characters.Handling; use Ada.Characters.Handling;
with Ada.Text_IO; use Ada.Text_IO;

package body ncurses2.getopt is

	opterr : Character := Character'Val(1);
	optopt : Character := '?';
	initialized : Boolean := False;

	nextchar : Natural := 0; 

	-- Ncurses doesn't use the non option elements so we are spared
	-- the job of computing those.

	-- also the user is not allowed to modify argv or argc
	-- Doing so is Erroneous execution.

	--  longoptions are not handled.

procedure qgetopt(
		retval : out Integer; 
		argc : Integer;
		argv : stringfunc; -- argv will be the Argument function.
		optstring : String;
		optind : in out Integer;  -- ignored for ncurses, must be initialized to 1 by the caller
		optarg : out stringa -- a garbage colector would be useful here.
		) is
		
	print_errors : Integer := Character'Pos(opterr);
	package BS is new Ada.Strings.Bounded.Generic_Bounded_Length(200);
	use BS; 
	optargx : Bounded_String;
begin
		
	if argc < optind then
		retval := -1;
		return;
	end if;
		
	optargx := To_Bounded_String("");
		
	if nextchar = 0 then
		
		if argv(optind) = "--" then 
			-- the rest are non-options, we ignore them
			retval := -1;
			return;
		end if;
		
		if argv(optind)(1) /= '-' or argv(optind)'Length = 1 then -- NONOPTION_P
			optind := optind + 1;
			optarg := new String'(argv(optind));
			retval := 1;
			return;
		end if;
		
		nextchar := 2; -- skip the one hyphen.
	end if;
		
	-- Look at and handle the next short option-character.
	declare
		c : Character := argv(optind)(nextchar);
		temp : Natural := Ada.Strings.Fixed.Index(
			optstring, String'(1 => c));
	begin
		if temp = 0 or c = ':' then
			Put_line(Standard_Error, 
				argv(optind) & ": invalid option -- " & c);
			optopt := c;
			c := '?';
			return;
		end if;
		
		if optstring(temp + 1) = ':' then
			if optstring(temp + 2) = ':' then
				-- This is an option that accepts an argument optionally.
				if nextchar /= argv(optind)'Length then
					optargx := To_Bounded_String(
						argv(optind)(nextchar .. argv(optind)'Length));
				else
					optarg := null;
				end if;
			else
				-- This is an option that requires an argument.
				if nextchar /= argv(optind)'Length then
					optargx := To_Bounded_String(
						argv(optind)(nextchar .. argv(optind)'Length));
					optind := optind + 1;
				elsif optind = argc then
					Put_line(Standard_Error, 
						argv(optind) & ": option requires an argument -- " & c);
					optopt := c;
					if optstring(1) = ':'  then
						c := ':';
					else
						c := '?';
					end if;
				else
					-- increment it again when taking next ARGV-elt as argument.
					optind := optind + 1;
					optargx := To_Bounded_String(argv(optind));
					optind := optind + 1;
				end if;
			end if;
			nextchar := 0;
		else -- no argument for the option
			if nextchar = argv(optind)'Length then
				optind := optind + 1;
				nextchar := 0;
			else
				nextchar := nextchar + 1;
			end if;
		end if;

		retval := Character'Pos(c);
		optarg := new String'( To_String( optargx));
		return;
	end;
end qgetopt;

end ncurses2.getopt;







